import os

try:
    from celery import Celery
    enabled = True
except ImportError:
    enabled = False
else:
    app = Celery(os.getenv('CELERY_APP_NAME') or 'unimatrix')
    app.config_from_object('django.conf:settings', namespace='CELERY')
    app.autodiscover_tasks()


    @app.task(bind=True)
    def debug_task(self):
        print('Request: {0!r}'.format(self.request))



"""Configures email-related Django settings."""
import os


__all__ = [
    'DEFAULT_FROM_EMAIL',
    'EMAIL_HOST',
    'EMAIL_HOST_PASSWORD',
    'EMAIL_HOST_USER',
    'EMAIL_PORT',
    'EMAIL_USE_TLS',
    'EMAIL_USE_SSL',
]

DEFAULT_FROM_EMAIL = os.getenv('DEFAULT_FROM_EMAIL') or 'root@localhost'

EMAIL_HOST = os.getenv('EMAIL_HOST') or 'localhost'

EMAIL_HOST_PASSWORD = os.getenv('EMAIL_HOST_PASSWORD') or ''

EMAIL_HOST_USER = os.getenv('EMAIL_HOST_USER')

EMAIL_PORT = int(os.getenv('EMAIL_PORT') or 0)

EMAIL_USE_TLS = os.getenv('EMAIL_USE_TLS') == '1'

EMAIL_USE_SSL = os.getenv('EMAIL_USE_SSL') == '1'


# Autodiscover the port from other settings
if not EMAIL_PORT:
    if EMAIL_USE_TLS:
        EMAIL_PORT = 587
    elif EMAIL_USE_SSL:
        EMAIL_PORT = 465
    else:
        EMAIL_PORT = 25


EMAIL_SSL_CERTFILE = os.getenv('EMAIL_SSL_CERTFILE')

EMAIL_SSL_KEYFILE = os.getenv('EMAIL_SSL_KEYFILE')



"""Blocks until all services required by the application are available."""
#pylint: disable=undefined-loop-variable
import logging
import time

from django.core.management.base import BaseCommand
from django.core.management.base import CommandError
from django.db import connections
from django.db.utils import OperationalError


class Command(BaseCommand):
    """Exposes a public API for the Django command framework to run the
    ``awaitservices`` command.
    """
    help = "Waits for services to become ready."
    logger = logging.getLogger('django')
    default_interval = 6.0
    max_attempts = 100

    def handle(self, *args, **options): #pylint: disable=unused-argument
        """Called when the ``awaitservices`` command is issued through
        ``manage.py``.
        """
        self.await_databases()

    def await_databases(self, interval=5.0):
        """Blocks until all databases are available.

        Args:
            interval (:obj:`float`): specifies the interval between checks,
                in seconds.

        Returns:
            :obj:`NoneType`
        """
        attempts = 0
        awaiting = {str(x) for x in connections}
        connected = set()
        while awaiting != connected:
            attempts += 1
            for alias in connections:
                connection = connections[alias]
                try:
                    cursor = connection.cursor()
                except OperationalError as exc:
                    msg = str.split(exc.args[0], '\n')[0]
                    self.logger.error(
                        "Database connection '%s' not ready (%s)",
                        alias, msg
                    )
                    continue
                self.logger.info("Database connection '%s' is operational",
                    alias)
                connected.add(alias)

            if attempts > self.max_attempts:
                raise CommandError("Connection '%s' not available after %s attempts"
                    % (alias, self.max_attempts))

            # If the databases were already up, check again here
            # so we don't wait unnecessarily.
            if awaiting == connected:
                break
            time.sleep(interval)



"""Declares :class:`EmailService`."""
from django.core.mail import send_mail
from django.conf import settings


class EmailService:
    """Provides an interface to send transactional emails."""

    def send(self, sender, subject, recipients, text, html):
        """Sends an email to the specified recipients."""
        if sender is None:
            sender = settings.DEFAULT_FROM_EMAIL
        return send_mail(subject, text, sender, recipients,
            html_message=html)
